/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef ALLLAYERS_H
#define ALLLAYERS_H

/* choice of decoding depth to be achieved */
typedef enum { dec_fields=0, dec_subsubbands, dec_subbands, dec_pcm } DecodeLevel;

#include "mpegPlatform.h"

#ifdef __cplusplus

#include <algorithm>
#include <cstdio>
#include <cmath>
#include <iostream>
#include <string.h>

// for reading file size
#include <sys/types.h>
#include <sys/stat.h>

using namespace std;

#define SBLIMIT      32   // max no subbands
#define SSLIMIT      18   // max no sub-subbands Layer 3
#define SBSSLIMIT    576  // SBLIMIT * SSLIMIT
#define SCALE_RANGE  64   // refers to last scalefactor

extern const double maaate_scalefactors[SCALE_RANGE];

#define HAN_SIZE     512

#ifndef PI
#define PI            3.14159265358979
#endif

//--------------------------------------------------------------------------

// abstract class to keep functions that are the same for all layers
class CSAPI_MPEG AllLayers {
public:

    // subband synthesis used by all layers
    void create_syn_filter(double filter[64][SBLIMIT]);
    void subband_syn      (double *bandptr, int ch, short *pcm);

    // layer-specific access to audio data
    virtual int    bitallocation   (unsigned int ch, unsigned int sb) = 0;
    virtual int    scfsi           (unsigned int ch, unsigned int sb) = 0;
    virtual float  scalefactor     (unsigned int ch, unsigned int sb, 
				    unsigned int ss) = 0;
    virtual int    sample          (unsigned int ch, unsigned int sb, 
			            unsigned int no, unsigned int ss) = 0;
    virtual double restored_sample (unsigned int ch, unsigned int sb, 
				    unsigned int no, unsigned int ss) = 0;
    virtual short  pcm_sample      (unsigned int ch, unsigned int sb, 
			            unsigned int no, unsigned int ss) = 0;

    // layer-specific handling of audio data
    virtual unsigned int calc_CRC () = 0;
    virtual bool         parse_data (DecodeLevel decode) = 0;

private:
};

//--------------------------------------------------------------------------

// inline operators

#else /* __cplusplus */

typdef struct AllLayers AllLayers;

#endif /* __cplusplus */

#endif
