/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef HEADER_H
#define HEADER_H

typedef enum { MPEG1, MPEG2 } Version;
typedef enum { I, II, III, RESERVED } Layer;
typedef enum { stereo, jstereo, dualch, singlech, multich} Mode;

#include "mpegPlatform.h"

#ifdef __cplusplus

#include <algorithm>
#include <cstdio>
#include <cmath>
#include <iostream>
#include <string.h>

// for reading file size
#include <sys/types.h>
#include <sys/stat.h>

using namespace std;

//--------------------------------------------------------------------------

// Class to handle mpeg audio frames' headers
class CSAPI_MPEG Header {
public:

  // access to header information and calculations thereof
  Version      version();
  Layer        layer();
  bool         crcprotected();
  int          bitrate();
  int          bitrateperchannel();
  int          bitrate_index();
  double       samplingrate();
  int          samplingrate_index();
  bool         padding();
  bool         extended();
  Mode         mode();
  const char*  mode_str();
  int          channels();
  int          mode_ext();
  const char*  mode_ext_str();
  unsigned int intensitystereobound();
  bool         copyright();
  bool         original();
  int          emphasis();
  unsigned int samples_per_frame();
  unsigned int framesize();
  unsigned int subbands();
  unsigned int check_sum();

  // print all header information
  void printheader ();
  void update_CRC (unsigned int data,
		   unsigned int length,
		   unsigned int *crc);

protected:
  struct FrameHeader {
    unsigned int sync            : 12;
    unsigned int version         : 1;
    unsigned int layer           : 2;
    unsigned int errorprotection : 1;
    unsigned int bitrate         : 4;
    unsigned int samplingrate    : 2;
    unsigned int padding         : 1;
    unsigned int extension       : 1;
    unsigned int mode            : 2;
    unsigned int mode_ext        : 2;
    unsigned int copyright       : 1;
    unsigned int original        : 1;
    unsigned int emphasis        : 2;
  };
  
  // 32 bit header 
  FrameHeader header;
  // 16 bit crc checksum
  unsigned int checksum : 16;

  bool checkheader ();
    
private:

  // only for caching
  int          c_bitrate;
  int          c_bitrateperchannel;
  int          c_channels;
  unsigned int c_intensitystereobound;
  unsigned int c_framesize;
  unsigned int c_subbands;

  int          p_bitrate();
  int          p_bitrateperchannel();
  int          p_channels();
  unsigned int p_intensitystereobound();
  unsigned int p_framesize();
  unsigned int p_subbands();

};

//--------------------------------------------------------------------------

#else

typedef struct Header Header;

#endif

#endif
