/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef LAYER1_H
#define LAYER1_H

#include "allLayers.H"
#include "MPEGfile.H"
#include "mpegPlatform.h"

using namespace std;

//--------------------------------------------------------------------------

// class definition
class CSAPI_MPEG Layer1 : public AllLayers {
public:
    Layer1(MPEGfile *mp);


    int    bitallocation (unsigned int ch, 
				 unsigned int sb);
    int    scfsi (unsigned int ch, unsigned int sb) {
      /* Layer 1 does not have any scfs information */
      return -1;
    };
    float  scalefactor (unsigned int ch, 
			       unsigned int sb, 
			       unsigned int ss=0);
    int    sample (unsigned int ch, 
			  unsigned int sb, 
			  unsigned int no, 
			  unsigned int ss=0);
    double restored_sample (unsigned int ch, 
				   unsigned int sb, 
				   unsigned int no, 
				   unsigned int ss=0);
    short  pcm_sample (unsigned int ch, 
			      unsigned int sb, 
			      unsigned int no, 
			      unsigned int ss=0);
    
    unsigned int calc_CRC();
    bool         parse_data(DecodeLevel decode);


private:
    MPEGfile *frame;

    // AUDIO DATA CONTENT:
    // channels()*subbands() bitallocations (4 bit to be read per entry)
    unsigned int allocation[2][SBLIMIT];
    // channels()*subbands() scale factor indexes (6 bit to be read per entry)
    unsigned int scale_index[2][SBLIMIT];
    // 12*channels()*subbands() sample data (2-15 bit to be read per entry)
    unsigned int samples[12][2][SBLIMIT];
    // 12*channels()*subbands() restored samples
    double restoredsamples[12][2][SBLIMIT];
    // 12*channels()*subbands() restored pcm samples
    short pcm[12][2][SBLIMIT];

    void decode_bitalloc();
    void decode_scale();
    void buffer_samples();
    void restore_samples();

    unsigned int bits_per_sample (unsigned int ch, unsigned int sb);


};

//--------------------------------------------------------------------------

#endif
