/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef LAYER2_H
#define LAYER2_H

#include "allLayers.H"
#include "MPEGfile.H"
#include "mpegPlatform.h"

using namespace std;

//--------------------------------------------------------------------------

// class definition
class CSAPI_MPEG Layer2 : public AllLayers {
public:
    Layer2 (MPEGfile *mp);
 
    int    bitallocation (unsigned int ch, unsigned int sb);
    int    scfsi (unsigned int ch, 
			 unsigned int sb);
    float  scalefactor (unsigned int ch, 
			       unsigned int sb, 
			       unsigned int no);
    int    sample (unsigned int ch,
			  unsigned int sb,
			  unsigned int no,
			  unsigned int ss);
    double restored_sample (unsigned int ch, 
				   unsigned int sb, 
				   unsigned int no,
				   unsigned int ss);
    short  pcm_sample (unsigned int ch, 
			      unsigned int sb, 
			      unsigned int no,
			      unsigned int ss);

    unsigned int calc_CRC();
    bool         parse_data(DecodeLevel decode);


private:
    MPEGfile *frame;

    // AUDIO DATA CONTENT:
    // channels()*subbands() bitallocations (2-4 bit to be read per entry)
    unsigned int allocation[2][SBLIMIT];
    // channels()*subbands() scale factor selections (2 bit per entry)
    unsigned int scfsis[2][SBLIMIT];
    // channels()*3*subbands() scale factor indexes (6 bit per entry)
    unsigned int scale_index[2][3][SBLIMIT];
    // 3*12*channels()*subbands() sample data (3-16 bit to be read per entry)
    unsigned int samples[12][2][3][SBLIMIT];

    // 12*channels()*subbands() restored samples
    double restoredsamples[12][2][3][SBLIMIT];
    // 12*channels()*subbands() restored samples
    short pcm[12][2][3][SBLIMIT];

    // subband allocation table structure for layer II
    struct SubbandTable {
      unsigned int    steps;
      unsigned int    bits;
      unsigned int    group;
      unsigned int    quant;
    };

    int table;
    SubbandTable alloctable[SBLIMIT][16];
    
    void pick_table();
    void read_bit_alloc();
    
    void decode_bitalloc();
    void decode_scale();
    void buffer_samples();
    void restore_samples();

    unsigned int grouping (unsigned int ch, unsigned int sb);
    unsigned int bits_per_sample (unsigned int ch, unsigned int sb);
    unsigned int bits_per_samplecode (unsigned int ch, unsigned int sb);


};

//--------------------------------------------------------------------------

#endif
