/*
    MPEG Maaate: An Australian MPEG audio analysis toolkit
    Copyright (C) 2000 Commonwealth Scientific and Industrial Research Organisation
    (CSIRO), Australia.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Here are defined useful functions used by modules */ 


#ifndef tools_H
#define tools_H

#include "pluginsPlatform.h"

/*---------------------- useful values -----------------------*/

#define RESTORED_SAMPLE_MAX 5.33333333331  
//maximum value of a restored sample for layer II

/*************************** other tools *************************/

/*-------------------------- search array ------------------------*/

CSAPI_PLUGINS
int search_array ( double tab[], double what, int lenght);
// return the position in tab of the nearer value of what,
// supposing that the array is sorted from min to max

/*************************** window functions ********************************/

CSAPI_PLUGINS
double square_window ( unsigned int upto, int x);
// square window from 0 to "upto"
/*
    1                _____________
                    |             |
    0  _____________|             |________________
                    0            upto  
*/

CSAPI_PLUGINS
double hamming_window ( unsigned int upto, int x);
// hamming window from 0 to upto
// 0.54 * 0.46 * cos (2 * Pi * x / upto)

CSAPI_PLUGINS
double welch_window ( unsigned int upto, int x);
// welch window from 0 to upto
// 1-((x-0.5 * upto) / (0.5 * upto))^2

CSAPI_PLUGINS
double bartlett_window ( unsigned int upto, int x);
// bartlett window from 0 to upto
// 1-|((x-0.5 * upto) / (0.5 * upto))|

#endif 
